from datetime import timedelta
from django.utils import timezone
from django.conf import settings
from django.contrib.sessions.backends.base import SessionBase, CreateError


def _store():
    return settings.MONGO_DB['session_store']


class SessionStore(SessionBase):
    '''
    MongoDB backend for Django sessions.
    '''

    def get_expiration_date(self):
        s = settings.SESSION_COOKIE_AGE
        return timezone.now() - timedelta(seconds=s)

    def load(self):
        mongo_session = _store().find_one({
            'session_key': self._get_or_create_session_key(),
            'creation_date': {
                '$gt': self.get_expiration_date()
            }
        })

        if mongo_session is None:
            # we just created a new session
            # self._session_key = None
            return {}
        else:
            return mongo_session['session_data']

    def exists(self, session_key):
        session = _store().find_one({
            'session_key': session_key,
        })

        if session is None:
            return False
        else:
            # mongodb ttl invalidation runs only once per minute, delete manually
            if session['creation_date'] <= self.get_expiration_date():
                self.delete(session_key)
                return self.exists(session_key)
            return True

    def create(self):
        while True:
            self._session_key = self._get_new_session_key()
            # ensure that session key is unique
            try:
                self.save(must_create=True)
            except CreateError:
                continue
            self.modified = True
            return

    def save(self, must_create=False):
        if self.session_key is None:
            return self.create()
        if must_create and self.exists(self.session_key):
            raise CreateError

        session = {
            'session_key': self.session_key,
            'session_data': self.encode(
                self._get_session(no_load=must_create)
            ),
            'creation_date': timezone.now()
        }

        _store().update_one(
            {'session_key': self.session_key},
            {'$set': session},
            upsert=True
        )

    def delete(self, session_key=None):
        if session_key is None:
            session_key = self._session_key
        if session_key is None:
            return
        _store().delete_one({'session_key': session_key})
        self._session_key = None

    def encode(self, session_dict):
        return session_dict

    def decode(self, session_data):
        return session_data

    def set_expiry(self, value):
        raise NotImplementedError



import sys
from os import path
from django.conf import settings
from pymongo import MongoClient

settings.configure(
    MONGO_DB = MongoClient()['mdbsession_test'],
    SESSION_ENGINE='mdbsession.session',
    SESSION_COOKIE_AGE=3,
)



import time
from unittest import TestCase
from django.conf import settings
from pymongo import MongoClient

from mdbsession.session import SessionStore


db = MongoClient()['mdbsession_test']


class TestSessionStore(TestCase):

    def setUp(self):
        # self.store = SessionStore()
        pass

    def tearDown(self):
        # clean up – remove database
        # print('Tested inside:', db.name)
        db.client.drop_database(db.name)

    def test_basic(self):
        store = SessionStore()
        self.assertFalse(store.modified)
        store['session_var'] = 'session_var_value'

        self.assertTrue(store.modified)
        self.assertEqual(store['session_var'], 'session_var_value')


    def test_we_can_save_session(self):
        store = SessionStore()
        store['var1'] = 'var1-value'
        store.save()

        self.assertIsNotNone(store.session_key)
        self.assertTrue(store.exists(store.session_key))


    def test_we_can_delete_session(self):
        store = SessionStore()
        store['var1'] = 'var1-value'
        store.save()
        store.delete()

        self.assertFalse(store.exists(store.session_key))
        self.assertIsNone(store.session_key)


    def test_we_can_flush_session(self):
        store = SessionStore()
        store['var1'] = 'var1-value'
        store.save()
        k = store.session_key
        store.flush()

        self.assertFalse(store.exists(k))
        self.assertIsNone(store.session_key)


    def test_we_can_store_and_retrieve_many_items(self):
        def make_pair(n):
            return 'bar-{}'.format(n), '{}-foo'.format(n)
        store = SessionStore()
        size = 12
        for n in range(12):
            k, v = make_pair(n)
            store[k] = v
        store.save()
        k = store.session_key
        store2 = SessionStore(session_key=k)
        store2.load()
        for n in range(12):
            k, v = make_pair(n)
            self.assertEqual(store2[k], v)


    def test_session_data_expires(self):
        store = SessionStore()
        store['key'] = 'value to expire'
        store.save()
        k = store.session_key
        self.assertTrue(store.exists(k))
        time.sleep(4)
        self.assertFalse(store.exists(k))



import setuptools
from os import path

with open("Readme.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='django-mdbsessions',
    version='0.0.2',
    author='somatoko',
    author_email='',
    description='Use MongoDB as Django session backend',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/somatoko/django-mdbsessions',
    zip_safe=False,
    packages=setuptools.find_packages(),
    install_requires=[
        'django >= 1.11',
        'pymongo >= 3.2.2'
    ],
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: Session',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='django mongodb session',
)
